#include "uogetopt.h"
#include "_stralloc.h"
#include "_error.h"
#include "_str.h"
#include "_ip4.h"
#include "_alloc.h"
#include "_buffer.h"
#include "_scan.h"
#include "fmt.h"
#include "getln.h"
#include "byte.h"
#include "open.h"
#include "strhash.h"
#include "bailout.h"
#include "interface.h"
#include "accorec.h"
#include "die_write.h"
#include "getclass.h"
#include "cdb.h"

static strhash_t ihash;
struct cdb cdb;

void
accorecord(const char *ipfrom,const char *ipto,
			unsigned long packets,unsigned long bytes)
{
	accorec_t *f;
	accorec_t *t;
	static stralloc cfrom;
	static stralloc cto;

	getclass(&cfrom,ipfrom);
	getclass(&cto,ipto);

	f=accorec_lookup(&ihash,cfrom.s,cfrom.len); /* XXX ipv4 */
	if (!f)
		f=accorec_new(&ihash,cfrom.s,cfrom.len,1);
	if (!f)
		oom();
	t=accorec_lookup(&ihash,cto.s,cto.len); /* XXX ipv4 */
	if (!t)
		t=accorec_new(&ihash,cto.s,cto.len,1);
	if (!t)
		oom();
	if (!accorec_add(t,ipfrom,4,packets,bytes)) 
		oom();
	if (!accorec_add(f,ipto,4,packets,bytes)) 
		oom();
}


static void dump(void)
{
	strhash_walkstart(&ihash);
	while (1) {
		int x;
		char *k;
		unsigned int kl;
		accorec_t *i;
		x=strhash_walk(&ihash,&k,&kl, (char **)&i,0);
		if (!x)
			break;
		accorec_print(k,kl,i);
	}
}

static uogetopt_t opts[]={
{'b',"bytes",UOGO_FLAG,&accorec_count_type, ACCOREC_COUNT_BYTES,
"Count bytes (default).",0},
{'l',"lines",UOGO_FLAG,&accorec_count_type, ACCOREC_COUNT_LINES,
"Count lines, not bytes.",0},
{'p',"packets",UOGO_FLAG,&accorec_count_type, ACCOREC_COUNT_PACKETS,
"Count packets, not bytes.",0},
{0,"",UOGO_MINARGS | UOGO_HIDDEN,0, 2 , 0,0},
{0,"",UOGO_MAXARGS | UOGO_HIDDEN,0, 2 , 0,0},
{'\0',0,UOGO_FLAG,0,0,0,0}
};
static const char *usagestring=
"usage: cacco-class-ip [options] CDB <in >out\n";
static const char *usagestring2=
" This program counts how may bytes were transferred from each class of\n"
" IP addresses the any IP address.\n"
" CDB is a constant database defining the classes.\n"
" The program may be unsuited for really large amounts of input data.\n";

int 
main(int argc, char **argv)
{
	int fd;
	bailout_progname(argv[0]);
	flag_bailout_fatal_begin=3;
	uogetopt(flag_bailout_log_name,PACKAGE,VERSION,
		&argc,argv,uogetopt_out,
		usagestring, opts,usagestring2);
	fd=open_read(argv[1]);
	if (fd==-1) xbailout(100,errno,"failed to open ",argv[1],0,0);
	cdb_init(&cdb,fd);
	accorec_print_name1=accorec_print_class;
	accorec_print_name2=accorec_print_ipv4;

	if (-1==strhash_create(&ihash,256,32,strhash_hash)) oom();
	readloop(0);
	dump();
	if (-1==buffer_flush(buffer_1)) die_write();
	exit(0);
}
